#include "brasilmobil.h"

#include <qlineedit.h>
#include <qlayout.h>
#include <qtable.h>
#include <qframe.h>
#include <qpushbutton.h>

#if EMBEDDED
#include <stdio.h>
#include <qstringlist.h>
#else
#include <cstdio>
#endif

BrasilMobil::BrasilMobil()
: QMainWindow()
{
	QWidget *w;
	QVBoxLayout *vbox;
	QHBoxLayout *hbox;
	QLineEdit *ed;
	QFrame *f;
	QPushButton *p;
	QWidget *img;

	w = new QWidget(this);
#if !EMBEDDED
	w->setFixedSize(240, 320);
#endif
	setCentralWidget(w);

	f = new QFrame(w);
	f->setFrameStyle(QFrame::Panel | QFrame::Sunken);

	ed = new QLineEdit(f);

	p = new QPushButton("Find", f);

	t = new QTable(0, 2, w);
	t->horizontalHeader()->setLabel(0, "Translation");
	t->horizontalHeader()->setLabel(1, "-");
	t->setColumnWidth(1, 20);
	t->setColumnStretchable(0, true);

	img = new QWidget(w);
	img->setFixedSize(240, 50);
#if EMBEDDED
	img->setBackgroundPixmap(QPixmap("brasil.png"));
#else
	img->setPaletteBackgroundPixmap(QPixmap("brasil.png"));
#endif

	hbox = new QHBoxLayout(f);
	hbox->add(ed);
	hbox->add(p);

	vbox = new QVBoxLayout(w);
	vbox->add(f);
	vbox->add(t);
	vbox->add(img);

	connect(ed, SIGNAL(textChanged(const QString&)), SLOT(slotFind(const QString&)));

#if !EMBEDDED
	setFixedSize(240, 320);
#endif
	setCaption("BrasilMobil");

#if !EMBEDDED
	show();
#else
	showFullScreen();
#endif
}

BrasilMobil::~BrasilMobil()
{
}

void BrasilMobil::slotFind(const QString& text)
{
	QString s;
	FILE *f;
	char buffer[1024];
	int i;
	QStringList dicts;

	if(text.length() < 2)
	{
		addResult(0, QString::null);
		return;
	}

	i = 0;

	dicts << "fd-deu-por";
	dicts << "fd-por-deu";

#if EMBEDDED
	for(QStringList::Iterator it = dicts.begin(); it != dicts.end(); it++)
#else
	for(QStringList::iterator it = dicts.begin(); it != dicts.end(); it++)
#endif
	{
		lang = (*it);

		f = popen(QString("dict -d %1 %2").arg((*it)).arg(text).latin1(), "r");
		if(f)
		{
			while(!feof(f))
			{
				fgets(buffer, sizeof(buffer), f);
				if(!feof(f))
				{
					s = buffer;
					s = s.left(s.length() - 1);
					addResult(i, s);
					i++;
				}
			}
			pclose(f);
		}
	}
}

void BrasilMobil::addResult(int i, const QString& s)
{
	QString x;
	QStringList l;
	bool disregard = false;
	static int k = 0;

	if(i == 0)
	{
		t->setNumRows(0);
		k = -1;
	}

	if(s.isEmpty()) disregard = true;
	else if(s.startsWith("From")) disregard = true;
	else if(s.contains("definition") && s.contains("found"))
	{
		if(!s.startsWith("No"))
		{
			disregard = true;
			founddef = true;
		}
	}

	if(s.isNull() || disregard) return;

	k++;
	if(k >= t->numRows()) t->setNumRows(k + 1);

	x = QString::fromUtf8(s);

	if(s.startsWith("   "))
	{
		l = l.split(",", x);
		k -= 1;
#if EMBEDDED
		for(QStringList::Iterator it = l.begin(); it != l.end(); it++)
#else
		for(QStringList::iterator it = l.begin(); it != l.end(); it++)
#endif
		{
			x = (*it);
			while(x.startsWith(" ")) x = x.right(x.length() - 1);
			k++;
			if(k >= t->numRows()) t->setNumRows(k + 1);
			t->setText(k, 0, x);
			t->setPixmap(k, 1, QPixmap(""));
		}
	}
	else if(s.startsWith("  ") && founddef)
	{
		while(x.startsWith(" ")) x = x.right(x.length() - 1);
		t->setText(k, 0, x);
		t->setPixmap(k, 1, QPixmap("def.png"));
		if(lang == "fd-deu-por")
			t->setPixmap(k, 0, QPixmap("de.png"));
		else
			t->setPixmap(k, 0, QPixmap("br.png"));
	}
	else if(s.startsWith("No definitions"))
	{
#ifdef EMBEDDED
		int pos;
		QString r;

		r = "No definitions found for ";
		pos = x.find(r);
		if(pos >= 0) x.remove(pos, r.length());
		r = ", perhaps you mean:";
		pos = x.find(r);
		if(pos >= 0) x.remove(pos, r.length());
#else
		x.replace("No definitions found for ", "");
		x.replace(", perhaps you mean:", "");
#endif
		t->setText(k, 0, x);
		t->setPixmap(k, 1, QPixmap("notfound.png"));
		if(lang == "fd-deu-por")
		{
			t->setPixmap(k, 0, QPixmap("de.png"));
			lang = "fd-por-deu";
		}
		else
		{
			t->setPixmap(k, 0, QPixmap("br.png"));
			lang = "fd-deu-por";
		}
		founddef = false;
	}
	else
	{
#ifdef EMBEDDED
		int pos;
		QString r;

		r = "fd-deu-por: ";
		pos = x.find(r);
		if(pos >= 0) x.remove(pos, r.length());
		r = "fd-por-deu: ";
		pos = x.find(r);
		if(pos >= 0) x.remove(pos, r.length());
#else
		x.replace("fd-deu-por: ", "");
		x.replace("fd-por-deu: ", "");
#endif
		t->setText(k, 0, x);
		t->setPixmap(k, 1, QPixmap("note.png"));
		if(lang == "fd-deu-por")
			t->setPixmap(k, 0, QPixmap("br.png"));
		else
			t->setPixmap(k, 0, QPixmap("de.png"));
	}
}

